@@ -1,5 +1,8 @@
 package Dancer2::Plugin::FormValidator::Config;
 
+use strict;
+use warnings;
+
 use Moo;
 use Carp;
 use Types::Standard qw(HashRef Bool);
@@ -29,7 +32,7 @@ has messages => (
     isa      => HashRef,
     required => 1,
     builder  => sub {
-        return shift->config->{messages} // {};
+        return $_[0]->config->{messages} // {};
     }
 );
 
@@ -38,7 +41,7 @@ has messages_validators => (
     isa      => HashRef,
     required => 1,
     builder  => sub {
-        return shift->messages->{validators} // {};
+        return $_[0]->messages->{validators} // {};
     }
 );
 
@@ -47,7 +50,7 @@ has ucfirst => (
     isa      => Bool,
     lazy     => 1,
     builder  => sub {
-        return shift->messages->{ucfirst} // 1;
+        return $_[0]->messages->{ucfirst} // 1;
     }
 );
 
@@ -56,7 +59,7 @@ has language => (
     isa      => NonEmptyStr,
     lazy     => 1,
     builder  => sub {
-        return shift->messages->{language} // 'en';
+        return $_[0]->messages->{language} // 'en';
     }
 );
 
